@@ -16,9 +16,10 @@ from scipy.stats import norm
 import sys
 from scipy.optimize import minimize
 import copy
-from multiprocessing import Pool
+from multiprocessing import Pool, cpu_count
 import scipy.sparse as sp
 import scipy.sparse.linalg as spln
+import time
 #===============================================================================
 # The Model
 #===============================================================================
@@ -32,20 +33,16 @@ def model(parm, priors, h, attrs, data, sd):
     parm : sequence
         The position of the model. Takes arbitrary parameters.
         
-    initial : dict
-        A dictionary with keys as names of parameters in ``parm`` and values
-        as a 4-item list. The first item is the guessed value for the parameter,
-        the second is a string identifying whether the prior is normal (``norm``)
-        or uniform (``unif``). The third and fourth are the uniform lower and
-        upper boundaries or the mean and standard deviation of the normal
-        distribution.
+    priors : list of prior classes
+        A list containing objects that are either of the Uniform(), Normal() or 
+        MultiNorm() class. These specify the prior information on each parameter.
         
     h : ``hod.HOD`` instance
         A fully realised instance is best as it may make updating it faster
         
     attrs : list
         A list of the names of parameters passed in ``parm``. Corresponds to the
-        keys of ``initial``.
+        names of the objects in ``priors``.
         
     data : array_like
         The measured correlation function.
@@ -59,7 +56,6 @@ def model(parm, priors, h, attrs, data, sd):
         The log likelihood of the model at the given position.
         
     """
-    # print "parm before: ", parm
     ll = 0
 
     for prior in priors:
@@ -67,7 +63,6 @@ def model(parm, priors, h, attrs, data, sd):
         if isinstance(prior, Uniform):
             index = attrs.index(prior.name)
             if parm[index] < prior.low or parm[index] > prior.high:
-                print "returning inf because " + prior.name + " is " + str(parm[index])
                 return -np.inf
 
         elif isinstance(prior, Normal):
@@ -81,15 +76,13 @@ def model(parm, priors, h, attrs, data, sd):
 
     # Rebuild the hod dict from given vals
     hoddict = {attr:val for attr, val in zip(attrs, parm)}
-
     h.update(**hoddict)
 
     # The logprob of the model
-    model = h.corr_gal.copy()  # data + np.random.normal(scale=0.1)
+    model = h.corr_gal.copy()
 
     ll += np.sum(norm.logpdf(data, loc=model, scale=sd))
 
-    print ll
     return ll
 
 def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10,
@@ -158,11 +151,7 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
     """
 
     if len(priors) == 0:
-        raise ValueError("initial must be at least length 1")
-
-    # ##NOW TRY A LAPLACE APPROXIMATION TO FIND THE MODE
-    # res = minimize()
-
+        raise ValueError("priors must be at least length 1")
 
     # Save which attributes are updatable for HOD as a list
     attrs = []
@@ -183,24 +172,16 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
                      'omegac_h2', 'h']:
             raise ValueError(a + " is not a valid variable for MCMC in HOD")
 
-    # Make sure hodkwargs is ok
-    if nthreads > 1:
-        numthreads = 0
-    else:
-        numthreads = 1
-    if 'NumThreads' in hodkwargs:
-        del hodkwargs['NumThreads']
+    hodkwargs.update({"ThreadNum":nthreads})
 
-    # Initialise the HOD object - use all available cpus for this
-    h = HOD(r=r, ThreadNum=nthreads, **hodkwargs)
+    # Initialise the HOD objec
+    h = HOD(r=r, **hodkwargs)
 
     # It's better to get a corr_gal instance then the updates could be faster
     # BUT because of some reason we need to hack this and do it in a map() function
     h = Pool(1).apply(create_hod, [h])
 
-    # Now update numthreads for MCMC parallelisation
-    h.update(ThreadNum=numthreads)
-
+    hodkwargs.update({"ThreadNum":nthreads})
     # Set guess if not set
     if len(guess) != len(attrs):
         guess = []
@@ -212,7 +193,6 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
             elif isinstance(prior, MultiNorm):
                 guess += prior.means.tolist()
 
-    # Get an array of initial values
     guess = np.array(guess)
 
     # Get an initial value for all walkers, around a small ball near the initial guess
@@ -254,15 +234,14 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
             f.write(header)
 
         if chunks is None:
-            chunks = 1
-        else:
-            if nsamples % chunks != 0:
-                print "Warning: nsamples must be a multiple of chunks, setting to 1"
-                chunks = 1
+            chunks = nsamples
+
+        start = time.time()
         for i, result in enumerate(sampler.sample(pos, iterations=nsamples)):
-            if i % chunks == 0:
+            if (i + 1) % chunks == 0:
+                print "Done ", 100 * nsamples / float(i + 1), " %. Time per sample: ", (time.time() - start) / ((i + 1) * nwalkers)
                 with open(filename, 'a') as f:
-                    np.savetxt(f, sampler.flatchain[-chunks:, :])
+                    np.savetxt(f, sampler.flatchain[(i + 1 - chunks):i, :])
 
     return sampler.flatchain, np.mean(sampler.acceptance_fraction)
 
